# -*- coding: utf-8 -*-
"""
Created on Thu Nov 19 14:26:43 2020

@author: margeaux.carter

Module with functions to download weather data from HPRCC, 
process the data, and produce datafiles formatted for use 
in NeDNR climate and groundwater modeling processes

updated 2/26/2021 to more function with WeatherObs class and 
incorporate suggestions from review
"""
import requests
import pandas as pd
import numpy as np
import os
from datetime import datetime,timedelta
import math
import matplotlib.pyplot as plt
import matplotlib as mpl

#############   Constants   ###########################################
base_url = 'http://data.rcc-acis.org/'

# Calibration factors for Calibrated Hargreaves Method
    # Variables AHG, BHG, CHG, DHG and EHG are monthly coefficients
    # for the latest calibration (April 2010) of the Hargreaves method.  The form of
    # the calibration equations is described below.  Coefficients from D Martin.
    # Modified Hargreaves Equation:  ETr = a + (b + c  ( - Long )^ d )  HF^e
hg_calib_dict = {'AHG':[2.8668E-01, 3.2598E-01, 2.5084E-03, -4.3735E-01,
                    -2.2547E+00, -2.8816E+00, -1.1546E+00, -9.1006E-01,
                    -1.7995E+00, -1.1648E+00, -1.7264E-01, 2.1856E-01],
                'BHG':[-1.2433E+00, -4.7887E-01, -1.5951E-01, 3.1692E-01, 
                        6.6228E-01, 9.6598E-01, 7.9071E-01, 4.7423E-01,
                        7.7119E-01, 1.4375E+00, 1.3079E-01, -1.8532E+00],
                 'CHG':[7.8045E-04, 5.0522E-04, 3.8256E-04, 2.3812E-04,
                        9.9555E-05, 8.9062E-05, 3.7218E-05, 1.2955E-04,
                        1.2795E-04, 2.3230E-04, 3.9630E-04, 1.0219E-03],
                 'DHG':[1.7653E+00, 1.7520E+00, 1.7721E+00, 1.7403E+00,
                        1.9925E+00, 1.9414E+00, 1.7712E+00, 1.7865E+00,
                        2.0049E+00, 1.7806E+00, 1.8123E+00, 1.7600E+00],
                 'EHG':[1.3168E+00, 1.3627E+00, 1.2052E+00, 1.2592E+00,
                        1.0322E+00, 1.0264E+00, 1.2211E+00, 1.2066E+00,
                        8.9205E-01, 8.4140E-01, 9.9760E-01, 1.2360E+00]}

gsc = 4.92 #used in Hargreaved calculations 

#############   Functions   ###########################################
def makedirs(workingfolder):
    '''
    purpose: to make consistent directories in which to store weather data
    input: working folder (string) where directories will be created
    returns directory names (list of strings)
    '''
    
    if(os.path.isdir(workingfolder)):
        dirs = [os.path.join(workingfolder,'CropSim_WeatherData'+
                         str(datetime.date(datetime.now())).replace('-','')),
            os.path.join(workingfolder,'CropSim_WeatherData'+
                     str(datetime.date(datetime.now())).replace('-',''),'RawData'),
            os.path.join(workingfolder,'CropSim_WeatherData'+
                     str(datetime.date(datetime.now())).replace('-',''),'QAQC'),
            os.path.join(workingfolder,'CropSim_WeatherData'+
                     str(datetime.date(datetime.now())).replace('-',''),'WEA')]

        for directory in dirs:
            try:
                os.mkdir(directory)

            except OSError:
                if os.path.isdir(directory):
                    print (directory.split('\\')[-1]+' directory exists.')
                else:
                    print ('Error making directory.')#expand?
            else:
                print ('Successfully created '+directory.split('\\')[-1]+' directory.')
    else:
        print('Specified workingfolder is not a valid directory')
    return dirs


def calc_ETrHG(lat,long,Date,Max_Temp,Min_Temp):
    '''
     purpose: calculate the Hargreaves Reference ET for a single date and location
     input:   lat (float), the latitude of point of interest
              long (float), the longitude of pointof interest
              Date (datetime object), the date of interest
              Max_Temp (float), the maximum temperature recorded at location on date of interest
              Min_Temp (float), the minimum temperature recorded at location on date of interest
    returns etrhg(float), the reference ET
    '''
    # RLAT is the latitude expressed in radians
    rlat = lat * math.pi/180.
    #convert temps from *F to *C
    DayThiC = (Max_Temp - 32.0)/1.8
    DayTloC = (Min_Temp - 32.0)/1.8
    
    #add more comments explaining variable names...
    dr = 1+0.033*math.cos(2*Date.dayofyear*math.pi/365.0)
    sd = 0.409*math.sin(2*Date.dayofyear*math.pi/365.0-1.39)
    
    ws = math.acos(-math.tan(rlat)*math.tan(sd))

    ra = (24/math.pi)*dr*gsc*(ws*math.sin(rlat)*math.sin(sd)+math.cos(rlat)*math.cos(sd)*math.sin(ws))

    hargf = .0023*math.sqrt(abs(DayThiC-DayTloC))*(.5*(DayThiC+DayTloC)+17.8)*ra/2.45

    if hargf < 0.001:
        hargf = 0.001

    # Form of the Hargreaves that was calibrated to computed reference
    # crop evapotranspiration using the Penman-Monteith equation for the
    # state wide calibration current as of April 2010.  ETRHG is in units of inches/day
    # ETr = a + (b + c  ( - Long )^ d )  HF^e

    etrhg = (hg_calib_dict['AHG'][Date.month-1]+ \
             (hg_calib_dict['BHG'][Date.month-1]+ \
              hg_calib_dict['CHG'][Date.month-1]* \
              ((-long)**hg_calib_dict['DHG'][Date.month-1])) \
             *hargf**hg_calib_dict['EHG'][Date.month-1])/25.4
    
    return etrhg

def check_missing(data,variable):
    missing = list(data.loc[data[variable].isnull()].Date.dt.date)
    mcount = 0
    rmlist = []
    if(len(missing)>1):
        for i in range(len(missing)-1):
            if((missing[i+1]-missing[i])==timedelta(days=1)):
                mcount = mcount+1
            else:
                if(mcount>0):
                    rmlist.append(mcount)
                    mcount = 0
    return missing, rmlist

def downloadlog_open(datafolder,method,sdate,edate,grid=''):
    '''
    purpose: to make a logfile to record data as it is downloaded
    input:  datafolder (string) where datalog will be created
            method, either 'StnData' to denote data taken at weather stations, 
                or 'GridData' for data from interpolated gridded datasets
            sdate ('yyyy-mm-dd') as the first date of the data to be downloaded
            edate ('yyyy-mm-dd') as the last date of the data to be downloaded
            grid (string) as the grid specification, if method is GridData
    returns logfile_name (string)
    '''
    logfile_name = str(datetime.date(datetime.now()))+'rawdata_log.txt'
    #check if logfile exists, and either open a new file or add on to old one
    if(os.path.isfile(os.path.join(datafolder,logfile_name))):
        logfile= open(os.path.join(datafolder,logfile_name),'a+')
    else:
        logfile= open(os.path.join(datafolder,logfile_name),'w+')
    #write header info    
    logfile.write('Data Accessed '+str(datetime.date(datetime.now()))+'\n' )
    logfile.write("Data retireved from Regional Climate System's Applied Climate Information System at "+base_url+'\n')
    logfile.write('Data retireved from '+method)
    if(method=='GridData'):
        logfile.write('grid id '+grid)
    logfile.write(' from '+sdate +' to '+ edate+'\n' )
    logfile.write('For further documentation, please go to http://www.rcc-acis.org/docs_webservices.html\n' ) 
    if((datetime.now()-datetime.fromisoformat(edate))<timedelta(days=180)):
        #if end date is < 6 months prior to download date, log that grid data may be subject to revision
        logfile.write('Last retrieved weather data observed less than 6 months from download date. Data may be subject to revision.')
    logfile.close()
    return logfile_name

def weaqaqclog_open(qaqcfolder,datafolder):
    '''
    purpose: to make a logfile to record QA/QC process
    input:  qaqcfolder (string) where qaqclog will be created
            datafolder (string) where data to be processed is located        
    returns logfile_name (string)
    '''
    logfile_name = str(datetime.date(datetime.now()))+'qaqc_log.txt'
    if(os.path.isfile(os.path.join(qaqcfolder,logfile_name))):
        logfile= open(os.path.join(qaqcfolder,logfile_name),'a+')
    else:
        logfile= open(os.path.join(qaqcfolder,logfile_name),'w+')
    #write header info
    logfile.write('Record of Raw Data QAQC process for Cropsim WEA inputs\n')
    logfile.write('Data Processed on '+str(datetime.date(datetime.now()))+'\n' )
    logfile.write('Raw Data found '+datafolder+'\n')
    logfile.write('----------------------------------------------------------------------------\n')
    logfile.close()
    return logfile_name

#############   Class, Functions   ###########################################
class WeatherObs:
    'Common base class for weather data used in pursuit of NeDNR water management activities'
    def __init__(self,locationname,method,sdate,edate,stn_number=None,loc=None,grid=None,elev = None,filename = None):
        self.locationname =locationname
        self.method = method
        self.sdate = sdate
        self.edate = edate
        self.stn_number = stn_number
        self.loc = loc #(lat,lon) pair
        self.elev = elev
        self.grid = grid
        if(filename is not None):
            self.filename = filename
        else:
            self.filename = locationname+method+'.csv' 
            
        
    def download(self,datafolder):
        if(self.method=='StnData'):
            if(self.stn_number==''):
                print('Please provide station number')
            else:
                api_params = {
                    'sID': str(self.stn_number),
                    'sdate': self.sdate,
                    'edate': self.edate,
                    'meta': 'valid_daterange,ll,name,state,elev,sids',#
                    'elems':'1,2,4'#1 = maxT, 2 = minT, 4 = precip, 
                }                  #10 = snowfall, 11 = snow_depth,

        elif(self.method=='GridData'):
            if(self.loc==None or self.grid==None):
                print('Please provide location and/or grid')
            else:
                api_params = {
                    'loc': (str(self.loc[1]) + ',' + str(self.loc[0])),
                    'sdate': self.sdate,
                    'edate': self.edate,
                    'grid': self.grid,
                    'meta':'ll,elev',#state, name not available from Grid
                    'elems': "1,2,4" # 1 = maxT, 2 = minT, 4 = precip
                }

        #download data from api        
        req = requests.get(base_url+self.method, api_params)
        
        #get metadata and apply to weather obs object
        print(req.json())

        metadata = req.json()['meta']
        #split lats/lons first
        if('ll' in metadata.keys()):
            [lon, lat] = metadata['ll']
            metadata['loc'] = (lat,lon)
            del metadata['ll']
        if('lat' in metadata.keys()):
            metadata['loc'] = (metadata['lat'],metadata['lon'])
            del metadata['lat'],metadata['lon']
            
        #assign metadata to weather object, individually by key
        newattr = []
        oldattr = []
        for key in metadata.keys():
            if key in self.__dict__.keys():
                oldattr.append(key) 
            else:
                newattr.append(key)
    
            self.__dict__[key]=metadata[key]
        if(len(oldattr)>0):
            print(self.__dict__['locationname']+' '+', '.join(oldattr)+' updated')    
        if(len(newattr)>0):
            print(self.__dict__['locationname']+' '+', '.join(newattr)+' added')  
              
        #get time series data and apply to weather obs object
        data = pd.DataFrame(req.json()['data'])
        data = data.rename(columns = {0:'Date',1:'Max_Temp',2:'Min_Temp',3:'Precip'})
        self.data=data
        #save data to datafolder 
        self.filename = self.locationname+self.method+'.csv'
        data.set_index('Date').to_csv(os.path.join(datafolder,self.filename))
        
        return

    def downloadlog_addto(self,datafolder,logfile_name):
        logfile= open(os.path.join(datafolder,logfile_name),'a+')
        logfile.write(self.locationname+'\n')
        for key in self.__dict__.keys():
            if(key!='data' and key!='locationname'):
                if(self.__dict__[key] is not None):
                    logfile.write(key+': '+str(self.__dict__[key])+'\n')
        logfile.write('\n')
        logfile.close() 
        return

    def getdata(self,datafolder):
        if(self.method=='old'):
            #to read in the old raw data text files
            #filename needs to be defined as naming system is inconsistent
            data = pd.read_csv(os.path.join(datafolder,self.filename),skipinitialspace=True,
                                        skiprows=2,na_values=['M','S','T'])
            data = data.rename(columns ={'MaxTemperature':'Max_Temp',
                    'MinTemperature':'Min_Temp',
                    'Precipitation':'Precip'})       
        else:
            data = pd.read_csv(os.path.join(datafolder,self.filename))
    
        #because sometimes these are read as strings for no particular reason
        data[['Max_Temp','Min_Temp','Precip']] = data[['Max_Temp','Min_Temp','Precip']].apply(pd.to_numeric, errors='coerce')
        data['Date'] = pd.to_datetime(data['Date'])
        #assign to class object
        self.data=data[['Date','Max_Temp','Min_Temp','Precip']]
        
        return
    
    def weaqaqclog_addto(self,qaqcfolder, logfile_name,N=0):
        writeWEA = True
    
        with open(os.path.join(qaqcfolder,logfile_name),'a+') as logfile:
            logfile.write('--------------------------------'+self.locationname+'--------------------------------\n')
            annual_errorcount = []
        
            for y in self.data.Date.dt.year.unique():
                count = 0
                unphys = []
                logfile.write(str(y)+'\n')
                if(np.any(self.data.loc[self.data.Date.dt.year==y,['Max_Temp','Min_Temp','Precip']].isnull())):
                    logfile.write('Data Missing \n')
                    missing, rmlist = check_missing(self.data.loc[self.data.Date.dt.year==y],'Max_Temp')
                    logfile.write('\tMaximum Temperature: ' +str(len(missing))+' day(s)\n')
                    if(len(rmlist)>0):
                        logfile.write('\t                     ' +str(len(rmlist))+' multi-day period(s)\n')
                        logfile.write('\t                     longest missing data period: '+str(max(rmlist))+' days\n')
                        if(max(rmlist)>2):
                            logfile.write('\t                     missing data periods > 2 days: '+str(sum([1 for c in rmlist if c>2]))+'\n')
        
                    missing, rmlist = check_missing(self.data.loc[self.data.Date.dt.year==y],'Min_Temp')
                    logfile.write('\tMinimum Temperature: ' +str(len(missing))+' day(s)\n')
                    if(len(rmlist)>0):
                        logfile.write('\t                     ' +str(len(rmlist))+' multi-day period(s)\n')
                        logfile.write('\t                     longest missing data period: '+str(max(rmlist))+' days\n')
                        if(max(rmlist)>2):
                            logfile.write('\t                     missing data periods > 2 days: '+str(sum([1 for c in rmlist if c>2]))+'\n')
        
                    missing, rmlist = check_missing(self.data.loc[self.data.Date.dt.year==y],'Precip')
                    logfile.write('\tPrecipitation:       ' +str(len(missing))+' day(s)\n')
                    if(len(rmlist)>0):
                        logfile.write('\t                     ' +str(len(rmlist))+' multi-day period(s)\n')
                        logfile.write('\t                     longest missing data period: '+str(max(rmlist))+' days\n')
                        if(max(rmlist)>2):
                            logfile.write('\t                     missing data periods > 2 days: '+str(sum([1 for c in rmlist if c>2]))+'\n')
                    count = count + max([self.data.loc[self.data.Date.dt.year==y,'Max_Temp'].isnull().sum(),
                                         self.data.loc[self.data.Date.dt.year==y,'Min_Temp'].isnull().sum(),
                                         self.data.loc[self.data.Date.dt.year==y,'Precip'].isnull().sum()])
                else:
                    logfile.write('No data missing in '+str(y)+'\n')
                               
                if(np.any(self.data.loc[self.data.Date.dt.year==y,'Max_Temp']<self.data.loc[self.data.Date.dt.year==y,'Min_Temp'])):
                    logfile.write('Days with Min Temp > Max Temp: '+str((self.data.loc[self.data.Date.dt.year==y,'Max_Temp']<
                                                     self.data.loc[self.data.Date.dt.year==y,'Min_Temp']).sum())+'\n')
                    count = count + (self.data.loc[self.data.Date.dt.year==y,'Max_Temp']<self.data.loc[self.data.Date.dt.year==y,'Min_Temp']).sum()
                else:
                    unphys.append('Min Temp < Max Temp')
    
                if(np.any(self.data.loc[self.data.Date.dt.year==y,'Max_Temp']<-50)):
                    logfile.write('Days with Max Temp < -50 *F: '+str((self.data.loc[self.data.Date.dt.year==y,'Max_Temp']<-50).sum())+'\n')
                    count = count + (self.data.loc[self.data.Date.dt.year==y,'Max_Temp']<-50).sum()
                else:
                    unphys.append('Max Temp > -50 *F')
            
            if(np.any(self.data.loc[self.data.Date.dt.year==y,'Max_Temp']>125)):
                logfile.write('Days with Max Temp > 125 *F: '+str((self.data.loc[self.data.Date.dt.year==y,'Max_Temp']>125).sum())+'\n')
                count = count + (self.data.loc[self.data.Date.dt.year==y,'Max_Temp']>125).sum()
            else:
                unphys.append('Max Temp < 125 *F \n')
    
            if(np.any(self.data.loc[self.data.Date.dt.year==y,'Min_Temp']<-50)):
                logfile.write('Days with Min Temp < -50 *F: '+str((self.data.loc[self.data.Date.dt.year==y,'Min_Temp']<-50).sum())+'\n')
                count = count + (self.data.loc[self.data.Date.dt.year==y,'Min_Temp']<-50).sum()
            else:
                unphys.append('Min Temp > -50 *F')
                             
            if(np.any(self.data.loc[self.data.Date.dt.year==y,'Min_Temp']>100)):
                logfile.write('Days with Min Temp > 100 *F: '+str((self.data.loc[self.data.Date.dt.year==y,'Min_Temp']>100).sum())+'\n')
                count = count + (self.data.loc[self.data.Date.dt.year==y,'Min_Temp']>100).sum()
            else:
                unphys.append('Min Temp < 100 *F')

            if(np.any(self.data.loc[self.data.Date.dt.year==y,'Precip']<0)):
                logfile.write('Days with Precip < 0 in: '+str((self.data.loc[self.data.Date.dt.year==y,'Precip']<0).sum())+'\n')
                count = count + (self.data.loc[self.data.Date.dt.year==y,'Precip']<0).sum()
            else:
                unphys.append('Precip >= 0 in')
            if(np.any(self.data.loc[self.data.Date.dt.year==y,'Precip']>10)):
                logfile.write('Days with Precip >10 in: '+str((self.data.loc[self.data.Date.dt.year==y,'Precip']>10).sum())+'\n')
                count = count + (self.data.loc[self.data.Date.dt.year==y,'Precip']>10).sum()
            else:
                unphys.append('Precip < 10 in')
            
            if(len(unphys)==7):
                logfile.write('All unphysical checks pass\n\n')
            elif(len(unphys)>0):
                logfile.write('All other unphysical checks pass\n\n')
                #logfile.write('For all days, all other unphysical checks pass, including: '+', '.join(unphys))
            else:
                logfile.write('\n')
        
            annual_errorcount.append(count)
        
    
        if any([count>N for count in annual_errorcount]):
            logfile.write('QAQC process found >= '+str(N)+' errors in an individual year for '+self.locationname+' data.\n')
            logfile.write('Please review '+self.locationname+' data separately before writing WEA files.\n\n')
            writeWEA = False
        logfile.close()
        
        return writeWEA
        
    
    def weaqaqc_makegraphs(self,qaqcfolder):
        try:
            os.mkdir(os.path.join(qaqcfolder,(self.locationname+self.method)))#make a folder for station/point
        except OSError:
            if os.path.isdir(os.path.join(qaqcfolder,(self.locationname+self.method))):
                #print(os.path.join(qaqcfolder,locationname).split('\\')[-1]+' directory exists.')
                pass
            else:
                print('Error making '+self.locationname+self.method+' directory.')
    
        for year in self.data.Date.dt.year.unique():
            fig = plt.figure(figsize = (20,8))
            gs = mpl.gridspec.GridSpec(1, 2, width_ratios=[3, 1]) 

            ax1 = fig.add_subplot(gs[0])
            ax1.plot(self.data.loc[(self.data.Date.dt.year == year),'Max_Temp'].values,'r.')
            ax1.plot(self.data.loc[(self.data.Date.dt.year == year),'Min_Temp'].values,'b.')
            ax1.set_title('Daily Weather Data', fontsize=14)
            ax1.set_ylabel(r'Temperature ($^\circ$F)', fontsize=14)
            ax1.set_xlabel('Day of Year', fontsize=14)
            ax1.legend(['Maximum Temperature','Minimum Temperature'],loc = 2, fontsize=14)
            ax2 = ax1.twinx()
            ax2.bar(self.data.loc[(self.data.Date.dt.year == year),'Date'].dt.dayofyear.values,
                             self.data.loc[(self.data.Date.dt.year == year),'Precip'].values,color = 'g')
            ax2.set_ylabel('Precipitation (in)', fontsize=14)
            ax2.legend(['Precip'],loc=1, fontsize=14)
        
            ax3 = fig.add_subplot(gs[1])
            ax3.barh(np.linspace(1,12,12),np.squeeze(self.data.loc[(self.data.Date.dt.year == year),
                                 ['Precip','Date']].groupby(self.data.Date.dt.month).sum().values)[::-1])
            ax3.set_title('Monthly Precipitation', fontsize=14)
            ax3.set_xlabel('Precipitation (in)', fontsize=14)
            #ax3.set_ylabel('Month', fontsize=14)
            ax3.set_yticks(np.linspace(1,12,12))
            ax3.set_yticklabels(['Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec'][::-1], fontsize=12)

            fig.suptitle('Weather Station: '+self.locationname+', '+str(year), fontsize=18, fontweight='bold')
            fig.savefig(os.path.join(qaqcfolder,(self.locationname+self.method),(self.locationname+str(year)+'.png')))
            plt.close()#to clear axis so figure won't show in jupyter
        print('QAQC graphs for '+self.locationname+' created.')
    
        return
 
    def weaqaqc_processdata(self):
        for d in self.data.index:
            #fill in missing data
            #Kari:  I've seen other stuff Flatwater has written where they set alt values for the first few days that get replaced
            #currently does not fail if initial days are missing, but sets them equal to set values. 
            if(np.isnan(self.data.loc[d,'Max_Temp'])):
                if(d>2):
                    self.data.loc[d,'Max_Temp'] = self.data['Max_Temp'].rolling(window=3).mean()[d-1]
                else:
                    #fills initial missing data with first not-missing value
                    self.data.loc[d,'Max_Temp'] = self.data.loc[self.data['Max_Temp'].notna(),'Max_Temp'].values[0]
                    #print('problem with initial Max Temp')
            if(np.isnan(self.data.loc[d,'Min_Temp'])):
                if(d>2):
                    self.data.loc[d,'Min_Temp'] = self.data['Min_Temp'].rolling(window=3).mean()[d-1]
                else:
                    #fills initial missing data with first not-missing value
                    self.data.loc[d,'Min_Temp'] = self.data.loc[self.data['Min_Temp'].notna(),'Min_Temp'].values[0]
                    #print('problem with initial Min Temp')
            if(np.isnan(self.data.loc[d,'Precip'])):
                if(d>2):
                    self.data.loc[d,'Precip'] = self.data['Precip'].rolling(window=3).mean()[d-1]
                else:
                    self.data.loc[d,'Precip'] = 0.0#probably an ok sub - consistent with previous method
                    #print('problem with initial Precip')
            
            #if Min>Max Temp, flip
            #could be done in one step like below, but need order right
            # data[['Min_Temp','MaxTemp']]=data.loc[data['Max_Temp']<data['Min_Temp'],['Max_Temp','Min_Temp']]
            if(self.data.loc[d,'Max_Temp']<self.data.loc[d,'Min_Temp']):
                self.data.loc[d,['Min_Temp','MaxTemp']]=self.data.loc[d,['Max_Temp','Min_Temp']]
    
            #correct extremes/unphysical values with smoothing
            # -50 < Max Temp < 125 *F
            if((self.data.loc[d,'Max_Temp']<-50) or (self.data.loc[d,'Max_Temp']>125)):
                self.data.loc[d,'Max_Temp'] = self.data.loc[d,'Max_Temp'].rolling(window=3).mean()[d]#should this be d or d-1?
            # -50 < Min Temp < 100 *F
            if((self.data.loc[d,'Min_Temp']<-50) or (self.data.loc[d,'Min_Temp']>100)):
                self.data.loc[d,'Min_Temp'] = self.data.loc[d,'Min_Temp'].rolling(window=3).mean()[d]  
            # 0 < Precip < 10 in 
            if((self.data.loc[d,'Precip']<0) or (self.data.loc[d,'Precip']>10)):
                self.data.loc[d,'Precip'] = self.data.loc[d,'Precip'].rolling(window=3).mean()[d]
        return self.data
    
    def wea_writefile(self,weafolder,processed_data):
        #confirm that input data has correct type
        data = processed_data[['Max_Temp','Min_Temp','Precip']].apply(pd.to_numeric, errors='coerce')
        data['Date'] = pd.to_datetime(processed_data['Date'])
        #calculate reference ET for printing to WEA file
        data['ETRHG']=[calc_ETrHG(self.loc[0],self.loc[1],data.loc[d,'Date'],
                                  data.loc[d,'Max_Temp'],data.loc[d,'Min_Temp']) for d in data.index]
    
        for year in self.data.Date.dt.year.unique():
            # opening file and writing headers to it
            with open(os.path.join(weafolder,(self.locationname+str(year)+'.WEA')),'w+') as f:
                if(self.method=='StnData' or self.method=='old'):
                    f.write('{:25}'.format(self.name)+
                            '{:4}'.format(self.state)+
                            '{:10}'.format(self.sids[0].replace(' ',''))+#'{:10}'.format(self.stn_number)
                            str(year)+'\n')
                if(self.method=='GridData'):
                    f.write('Gridded Data for '+
                            '{:8}'.format(self.locationname)+
                            'Grid: '+'{:8}'.format(str(self.grid))+
                            str(year)+'\n')
                f.write('{:3}'.format(data.loc[0,'Date'].month)+
                        '{:3}'.format(data.loc[0,'Date'].day)+
                        '{:6}'.format(data.loc[data.Date.dt.year == year][-1:].Date.dt.dayofyear.values[0])+
                        '   LAT='+'{:7.2f}'.format(self.loc[0])+ 
                        '   LONG='+'{:8.2f}'.format(self.loc[1])+
                        '   ELEV(ft)='+'{:8.0f}'.format(self.elev)+'.\n'+
                        'DOY,    TMAX,    TMIN,  PRECIP,    EVAP,   ETRHG\n')
                # create a dataframe and append after headers
                #this is driving me nuts right now...
                df = pd.DataFrame({
                        'DOY':data.loc[data.Date.dt.year == year].Date.dt.dayofyear.astype('str').str.pad(3,side='left'),
                        'TMAX':data.loc[data.Date.dt.year == year].Max_Temp.round(1).map('{:8.1f}'.format),
                        'TMIN':data.loc[data.Date.dt.year == year].Min_Temp.round(1).map('{:8.1f}'.format),
                        'PRECIP':data.loc[data.Date.dt.year == year].Precip.map('{:8.2f}'.format),
                        'EVAP':['  -99.00']*len(data.loc[data.Date.dt.year == year]),
                        'ETRGH':data.loc[data.Date.dt.year == year].ETRHG.round(2).map('{:8.2f}'.format)
                        }).set_index('DOY')
                df.to_csv(os.path.join(weafolder,(self.locationname+str(year)+'.WEA')),mode='a',header=False)
        print('WEA files for '+self.locationname+' created.\n')
        return
    
    def wea_doall(self,datafolder,qaqcfolder,weafolder,downloadlogfile,qaqclogfile):
        #download the data and add to logfile
        self.download(datafolder)
        self.downloadlog_addto(datafolder,downloadlogfile)
        #retireve the data
        self.getdata(datafolder)
        #perform qaqc processes and log
        self.weaqaqc_makegraphs(qaqcfolder)
        self.weaqaqclog_addto(qaqcfolder, qaqclogfile)
        processed_data = self.weaqaqc_processdata()
        #write the wea file
        self.wea_writefile(weafolder,processed_data)
        return

